@@ -26,12 +26,14 @@ from .runners import (
     Firefox,
     SessionWorker,
     DefaultEditor,
+    ImageMagick,
     DefaultRunners,
 )
 from .site import (
     Synchronize,
     WorkingSites, ExchangeSites,
     MinioFileUploader, MinioFileDownloader,
+    RequestRevisionsAs,
 )
 from .file import TrackedFile, Revision
 from .kitsu import KitsuAPIWrapper
@@ -83,11 +85,18 @@ class LoginPageWidget(CustomPageWidget):
             kwargs={"kitsu_id": kitsu_id, "kitsu_password": kitsu_password},
         )
 
-        if not logged_in:
-            self.error_label.setText(
-                "<font color=#C71B1F>Login failed... \
-                Check credentials !</font>"
-            )
+        if logged_in:
+            if logged_in == 1:
+                text = "<font color=#C71B1F>Login failed... Check credentials !</font>"
+            elif logged_in == 2:
+                support = self.session.cmds.Flow.get_value(self.project_oid + "/admin/project_settings/support")
+                text = "<font color=#D66700>A user with the same name but a different Kitsu ID is already registered for this project."
+                if support:
+                    text += f"<br>We invite you to contact the support of the project ({support}).</font>"
+                else:
+                    text += "</font>"
+            
+            self.error_label.setText(text)
             return
 
         self.error_label.setText("")
@@ -177,126 +186,6 @@ class WorkingSiteChoiceValue(flow.values.ChoiceValue):
         return self.root().project().get_working_sites().mapped_names()
 
 
-class RequestRevisionsAs(flow.Action):
-
-    pattern = flow.SessionParam("").watched().ui(
-        placeholder="Revision oid pattern"
-    )
-    requesting_site_name = flow.Param("default", WorkingSiteChoiceValue).watched().ui(
-        label="Requesting site"
-    )
-    requested_site_name = flow.Param("default", WorkingSiteChoiceValue).watched().ui(
-        label="Site to query"
-    )
-    revision_oids = flow.SessionParam("").ui(
-        editor="textarea",
-        html=True,
-        editable=False,
-    )
-
-    def allow_context(self, context):
-        current_site = self.root().project().get_current_site()
-        return context and current_site.request_files_from_anywhere.get()
-    
-    def child_value_changed(self, child_value):
-        if child_value in (self.requesting_site_name, self.requested_site_name):
-            msg = "<h2>Request revisions</h2>"
-            if self.requested_site_name.get() == self.requesting_site_name.get():
-                msg += (
-                    "<font color=#D5000D>"
-                    "Requested and requesting sites can't be the same."
-                    "</font>"
-                )
-
-            self.message.set(msg)
-        elif child_value is self.pattern:
-            self._oids = self.glob(self.root().project().oid(), self.pattern.get(), 0)
-            self.revision_oids.set("<br>".join(self._oids))
-
-    def get_buttons(self):
-        self.requested_site_name.set(self.requested_site_name.choices()[0])
-        self.revision_oids.set("")
-        self._oids = None
-
-        return ["Request", "Close"]
-    
-    def ls(self, root_oid):
-        related_info, mapped_names = self.root().session().cmds.Flow.ls(root_oid)
-        relation_oids = [rel_info[0] for rel_info in related_info]
-        mapped_oids = ["%s/%s" % (root_oid, name) for name in mapped_names]
-        
-        return relation_oids + mapped_oids
-    
-    def get_last_publication(self, file_oid):
-        o = self.root().get_object(file_oid)
-
-        if not isinstance(o, TrackedFile):
-            return None
-        
-        head = o.get_head_revision()
-        
-        if not head:
-            return None
-        
-        return head.name()
-    
-    def glob(self, root_oid, pattern, level):
-        if level >= pattern.count("/") - 1:
-            return [root_oid]
-
-        matches = []
-        level_pattern = "/".join(pattern.split("/")[:level + 3])
-
-        if level_pattern.endswith("[last]"):
-            file_oid = self.root().session().cmds.Flow.resolve_path(root_oid + "/../..")
-            head_name = self.get_last_publication(file_oid)
-
-            if not head_name:
-                return []
-            
-            pattern = pattern.replace("[last]", head_name)
-            level_pattern = level_pattern.replace("[last]", head_name)
-        
-        for oid in self.ls(root_oid):
-            if fnmatch.fnmatch(oid, level_pattern):
-                matches += self.glob(oid, pattern, level + 1)
-        
-        return matches
-    
-    def run(self, button):
-        if button == "Close":
-            return
-        
-        if self._oids is None:
-            self._oids = self.glob(self.root().project().oid(), self.pattern.get(), 0)
-
-        objects = [self.root().get_object(oid) for oid in self._oids]
-        processed_oids = []
-        
-        for o in objects:
-            oid = o.oid()
-
-            if isinstance(o, Revision):
-                requested_site_name = self.requested_site_name.get()
-                requesting_site_name = self.requesting_site_name.get()
-
-                if not requesting_site_name in o.request_as.requesting_site_name.choices():
-                    oid += f" <font color='red'>Available on {requesting_site_name}</font>"
-                elif not requested_site_name in o.request_as.requested_site_name.choices():
-                    oid += f" <font color='red'>Not available on {requested_site_name}</font>"
-                else:
-                    o.request_as.run(None)
-                    oid += " <font color='green'>OK</font>"
-            else:
-                oid += " <font color='red'>Not a file revision</font>"
-            
-            processed_oids.append(oid)
-        
-        self.revision_oids.set("<br>".join(processed_oids))
-
-        return self.get_result(close=False)
-
-
 class Synchronization(flow.Object):
     ICON = ("icons.libreflow", "sync_arrow")
     
@@ -366,6 +255,7 @@ class KitsuConfig(flow.Object):
 class ProjectSettings(flow.Object):
     project_nice_name = flow.Param()
     project_thumbnail = flow.Param().ui(editor='textarea')
+    support = flow.Param("")
 
     non_editable_files = flow.Param("")
 
@@ -574,35 +464,40 @@ class Project(flow.Object):
         kitsu_api = self.kitsu_api()
 
         self.update_kitsu_host(self.admin.kitsu.server_url.get())
-        success = kitsu_api.log_in(kitsu_id, kitsu_password)
+        
+        if not kitsu_api.log_in(kitsu_id, kitsu_password):
+            return 1
 
-        if success:            
-            # Set current user
-            user_name = self.admin.users.get_user_name(kitsu_id)
-            
-            if not user_name:
-                # Create user profile if not registered
-                user = self.admin.users.add_user(kitsu_id)
-                user_name = user.name()
-            
-            self.set_user(user_name)
+        # Set current user
+        user_name = self.admin.users.get_user_name(kitsu_id)
+        
+        if not user_name:
+            # Create user profile if not registered
+            user = self.admin.users.add_user(kitsu_id)
 
-            # Create user folder at first login
-            user_settings_folder = self.user_settings_folder()
-            if not os.path.exists(user_settings_folder):
-                os.makedirs(user_settings_folder)
+            if not user:
+                return 2
 
-            # Save authentification tokens
-            tokens = kitsu_api.get_tokens()
-            kitsu_config = {}
-            kitsu_config["tokens"] = tokens
-            kitsu_config["kitsu_host"] = kitsu_api.get_host()
+            user_name = user.name()
+        
+        self.set_user(user_name)
 
-            if user_settings_folder is not None:
-                with open("%s/kitsu_config.json" % user_settings_folder, "w+") as f:
-                    json.dump(kitsu_config, f)
+        # Create user folder at first login
+        user_settings_folder = self.user_settings_folder()
+        if not os.path.exists(user_settings_folder):
+            os.makedirs(user_settings_folder)
 
-        return success
+        # Save authentification tokens
+        tokens = kitsu_api.get_tokens()
+        kitsu_config = {}
+        kitsu_config["tokens"] = tokens
+        kitsu_config["kitsu_host"] = kitsu_api.get_host()
+
+        if user_settings_folder is not None:
+            with open("%s/kitsu_config.json" % user_settings_folder, "w+") as f:
+                json.dump(kitsu_config, f)
+
+        return 0
     
     def log_out(self):
         self.kitsu_api().log_out()
@@ -688,6 +583,7 @@ class Project(flow.Object):
         self._RUNNERS_FACTORY.ensure_runner_type(MinioFileDownloader)
         self._RUNNERS_FACTORY.ensure_runner_type(SessionWorker)
         self._RUNNERS_FACTORY.ensure_runner_type(DefaultEditor)
+        self._RUNNERS_FACTORY.ensure_runner_type(ImageMagick)
 
     def ensure_runners_loaded(self):
         session = self.root().session()
