@@ -11,6 +11,7 @@ import hashlib
 import timeago
 import zipfile
 import fnmatch
+import subprocess
 
 import kabaret.app.resources as resources
 from kabaret import flow
@@ -22,6 +23,7 @@ from .maputils import SimpleCreateAction, ClearMapAction
 from .site import SyncMap, Request, RequestAs
 from .runners import LaunchSessionWorker, CHOICES, CHOICES_ICONS
 
+from libreflow.utils.b3d import wrap_python_expr
 pyversion = sys.version_info
 
 
@@ -196,7 +198,14 @@ class PublishFileAction(LaunchSessionWorker):
         return self.oid()
 
     def launcher_exec_func_name(self):
-        return "upload_published_revision"
+        return "process_after_publish"
+    
+    def process_after_publish(self):
+        if self.upload_after_publish.get():
+            self.upload_published_revision()
+        
+        if self._file.format.get() == 'blend':
+            self.save_blend_dependencies()
     
     def upload_published_revision(self):
         head = self._file.get_head_revision()
@@ -211,6 +220,22 @@ class PublishFileAction(LaunchSessionWorker):
         )
         process_jobs_action = self.root().project().admin.process_jobs
         process_jobs_action._process(upload_job)
+    
+    def save_blend_dependencies(self):
+        from blender_asset_tracer import trace, bpathlib
+        from pathlib import Path
+        import collections
+
+        head = self._file.get_head_revision()
+        path = Path(head.get_path())
+        report = collections.defaultdict(list)
+
+        for usage in trace.deps(path):
+            filepath = usage.block.bfile.filepath.absolute()
+            asset_path = str(usage.asset_path).replace('//', '')
+            report[str(filepath)].append(asset_path)
+        
+        head.dependencies.set(dict(report))
 
     def run(self, button):
         if button == "Cancel":
@@ -232,8 +257,7 @@ class PublishFileAction(LaunchSessionWorker):
 
         self._file.touch()
 
-        if self.upload_after_publish.get():
-            super(PublishFileAction, self).run(None)
+        super(PublishFileAction, self).run(None)
 
 
 class PublishFileFromWorkingCopy(PublishFileAction):
@@ -289,6 +313,7 @@ class Revision(flow.Object):
 
     site = flow.Param()
     sync = flow.Child(SyncMap).ui(expanded=True)
+    ready_for_sync = flow.BoolParam(True)
 
     open = flow.Child(OpenRevision)
     make_current = flow.Child(MakeCurrentRevisionAction)
@@ -298,6 +323,7 @@ class Revision(flow.Object):
     request_as = flow.Child(RequestAs)
     compute_hash_action = flow.Child(ComputeRevisionHash)
     check_hash = flow.Child(CheckRevisionHash)
+    dependencies = flow.Param("").ui(editor='textarea', editable=False)
 
     def get_path(self):
         return os.path.join(self.path.get(), self.file_name.get())
@@ -453,7 +479,7 @@ class Revisions(flow.Map):
 
         return columns
 
-    def add(self, name=None, object_type=None):
+    def add(self, name=None, object_type=None, ready_for_sync=True):
         publication_count = sum(
             list(map(lambda rev: int(not rev.is_working_copy()), self.mapped_items()))
         )
@@ -465,6 +491,7 @@ class Revisions(flow.Map):
         rev.user.set(self.root().project().get_user())
         rev.date.set(time.time())
         rev.site.set(self.root().project().get_current_site().name())
+        rev.ready_for_sync.set(ready_for_sync)
 
         return rev
 
@@ -1140,7 +1167,7 @@ bpy.ops.lfs.playblast(do_render=True, filepath='%s', studio='%s', project='%s',
             "--addons",
             "mark_sequence",
             "--python-expr",
-            python_expr,
+            wrap_python_expr(python_expr),
         ]
 
     def run(self, button):
@@ -1258,7 +1285,7 @@ class TrackedFile(File):
     current_user_sees = flow.Computed()
 
     show_history = flow.Child(GotoHistory)
-    publish_action = flow.Child(PublishFileAction).ui(label="Publish")
+    publish_action = flow.Child(PublishFileAction).injectable().ui(label="Publish")
     create_working_copy_action = flow.Child(CreateWorkingCopyAction).ui(
         label="Create working copy"
     )
@@ -1426,7 +1453,7 @@ class TrackedFile(File):
 
             revisions.remove(user_name)
 
-        working_copy = revisions.add(user_name)
+        working_copy = revisions.add(user_name, ready_for_sync=False)
         working_copy.set_sync_status("Available")
 
         try:
@@ -1468,9 +1495,9 @@ class TrackedFile(File):
 
         return working_copy
 
-    def publish(self, revision_name=None, source_path=None, comment="", keep_editing=False):
+    def publish(self, revision_name=None, source_path=None, comment="", keep_editing=False, ready_for_sync=True):
         revisions = self.get_revisions()
-        head_revision = revisions.add(revision_name)
+        head_revision = revisions.add(revision_name, ready_for_sync=ready_for_sync)
         head_revision.comment.set(comment)
         head_revision_path = head_revision.get_path()
         head_revision.set_sync_status("Available")
@@ -1686,11 +1713,17 @@ class FileRevisionNameChoiceValue(flow.values.ChoiceValue):
     
     def revert_to_default(self):
         source_file = self.get_file()
-        if source_file:
-            head = source_file.get_head_revision(sync_status="Available")
-            self.set(head.name() if head else None)
-        else:
+        
+        if not source_file:
             self.set(None)
+            return
+        
+        revision = source_file.get_head_revision(sync_status="Available")
+
+        if not revision:
+            revision = source_file.get_working_copy()
+        
+        self.set(revision.name() if revision else None)
 
 
 class FileRefRevisionNameChoiceValue(FileRevisionNameChoiceValue):
@@ -1736,6 +1769,7 @@ class PublishIntoFile(PublishFileAction):
 
     def get_buttons(self):
         self.message.set("<h2>Publish from an existing file</h2>")
+        self.target_file.set(None)
         self.source_file.set(self._file.display_name.get())
         self.source_revision_name.revert_to_default()
 
@@ -1756,22 +1790,22 @@ class PublishIntoFile(PublishFileAction):
     def check_file(self, file):
         expected_format = self._file.format.get()
         msg = "<h2>Publish from an existing file</h2>"
+        error_msg = ""
 
-        if not file or file.format.get() != expected_format:
-            self.message.set((
-                f"{msg}<font color=#D5000D>"
-                f"Target file must be in {expected_format} format."
-                "</font>"
-            ))
-            return False
+        if not file:
+            error_msg = "A target file must be set."
+        elif file.format.get() != expected_format:
+            error_msg = f"Target file must be in {expected_format} format."
         elif not self.source_revision_name.choices():
-            self.message.set((
-                f"{msg}<font color=#D5000D>"
-                f"Target file has no revision available on current site."
-                "</font>"
-            ))
+            error_msg = f"Target file has no revision available on current site."
+        
+        if error_msg:
+            self.message.set(
+                f"{msg}<font color=#D5000D>{error_msg}</font>"
+            )
             return False
         
+        self.message.set(msg)
         return True
     
     def check_revision_name(self, name):
@@ -1792,7 +1826,6 @@ class PublishIntoFile(PublishFileAction):
             return False
         
         self.message.set(msg)
-
         return True
     
     def child_value_changed(self, child_value):
@@ -1802,20 +1835,16 @@ class PublishIntoFile(PublishFileAction):
             self.check_file(self.target_file.get())
             self.check_revision_name(self.source_revision_name.get())
         elif child_value is self.source_revision_name:
-            self.revision_name.set(self.source_revision_name.get())
+            value = self.source_revision_name.get()
+            self.revision_name.set(value)
             self.comment.set("Created from %s (%s)" % (
                 self._file.display_name.get(),
-                self.source_revision_name.get(),
+                value,
             ))
         elif child_value is self.revision_name:
             revision_name = self.revision_name.get()
             self.check_revision_name(revision_name)
 
-            self.comment.set("Created from %s (%s)" % (
-                self._file.display_name.get(),
-                revision_name,
-            ))
-
     def run(self, button):
         if button == "Cancel":
             return
@@ -1832,7 +1861,8 @@ class PublishIntoFile(PublishFileAction):
         if not self.check_revision_name(revision_name):
             return self.get_result(close=False)
         
-        source_revision = self._file.get_revision(revision_name)
+        source_revision_name = self.source_revision_name.get()
+        source_revision = self._file.get_revision(source_revision_name)
         
         # Publish in target file
         target_file.lock()
@@ -1867,6 +1897,7 @@ class CreateWorkingCopyFromFile(CreateWorkingCopyBaseAction):
 
     def get_buttons(self):
         msg = "<h2>Create working copy from another file</h2>"
+        self.source_file.set(None)
         self.target_file.set(self._file.display_name.get())
 
         if self._file.has_working_copy(from_current_user=True):
@@ -1874,8 +1905,10 @@ class CreateWorkingCopyFromFile(CreateWorkingCopyBaseAction):
                 "<font color=#D66700>"
                 "You already have a working copy on %s. "
                 "Creating a working copy will overwrite the current one."
-                "</font>" % self._file.display_name.get()
+                "</font><br>" % self._file.display_name.get()
             )
+        else:
+            msg += "<br>"
         
         self.message.set(msg)
 
@@ -1891,31 +1924,32 @@ class CreateWorkingCopyFromFile(CreateWorkingCopyBaseAction):
     def check_file(self, file):
         expected_format = self._file.format.get()
         msg = "<h2>Create working copy from another file</h2>"
-
-        if not file or file.format.get() != expected_format:
-            self.message.set((
-                f"{msg}<font color=#D5000D>"
-                f"Source file must be in {expected_format} format."
-                "</font>"
-            ))
-            return False
-        elif not self.source_revision_name.choices():
-            self.message.set((
-                f"{msg}<font color=#D5000D>"
-                f"Source file has no revision available on current site."
-                "</font>"
-            ))
-            return False
+        error_msg = ""
 
         if self._file.has_working_copy(from_current_user=True):
             msg += (
                 "<font color=#D66700>"
                 "You already have a working copy on %s. "
                 "Creating a working copy will overwrite the current one."
-                "</font>" % self._file.display_name.get()
+                "</font><br>" % self._file.display_name.get()
             )
+        else:
+            msg += "<br>"
 
-        self.message.set(msg)
+        if not file:
+            error_msg = "A source file must be set."
+        elif file.format.get() != expected_format:
+            error_msg = f"Source file must be in {expected_format} format."
+        elif not self.source_revision_name.choices():
+            error_msg = f"Source file has no revision available on current site."
+        
+        if error_msg:
+            self.message.set(
+                f"{msg}<font color=#D5000D>{error_msg}</font>"
+            )
+            return False
+
+        self.message.set(msg + "<br><br>")
         
         return True
     
@@ -2042,10 +2076,99 @@ class OpenTrackedFolderAction(RevealInExplorer):
         self._folder.get_seen_revision().open.run(None)
 
 
+class FolderAvailableRevisionName(FileRevisionNameChoiceValue):
+
+    action = flow.Parent()
+
+    def get_file(self):
+        return self.action._folder
+
+
+class ResizeTrackedFolderImages(RunAction):
+    '''
+    Computes half-resized versions of all PNG images contained in a source tracked folder
+    in another tracked folder suffixed with `_half`.
+    '''
+    _folder = flow.Parent()
+    _files = flow.Parent(2)
+    revision_name = flow.Param(None, FolderAvailableRevisionName).watched()
+    publish_comment = flow.SessionParam("")
+
+    def runner_name_and_tags(self):
+        return 'ImageMagick', []
+    
+    def extra_argv(self):
+        in_pattern = '{}/*.png[{}]'.format(self._source_folder_path, self._resize_format)
+        out_pattern = '{}/%[filename:base].png'.format(self._target_folder_path)
+        return ['convert', in_pattern, '-set', 'filename:base', '%[basename]', out_pattern]
+    
+    def get_buttons(self):
+        self.message.set((
+            "<h2>Resize images in {0}</h2>"
+            "Every PNG image included in this folder will have a resized version placed in the <b>{0}_half</b> folder.".format(
+                self._folder.name()
+            )
+        ))
+        self.revision_name.revert_to_default()
+
+        return ['Resize images', 'Cancel']
+    
+    def child_value_changed(self, child_value):
+        if child_value is self.revision_name:
+            self.publish_comment.set(
+                "Half-resized images from %s folder" % self._folder.name()
+            )
+    
+    def _get_image_dimensions(self, img_path):
+        exec_path = self.root().project().admin.user_environment['IMAGEMAGICK_EXEC_PATH'].get()
+        
+        dims = subprocess.check_output([exec_path, 'convert', img_path, '-format', '%wx%h', 'info:'])
+        dims = dims.decode('UTF-8').split('x')
+
+        return tuple(map(int, dims))
+    
+    def run(self, button):
+        if button == 'Cancel':
+            return
+
+        # Setup target folder
+        target_folder_name = self._folder.name() + '_half'
+        
+        if not self._files.has_mapped_name(target_folder_name):
+            self._files.create_folder.folder_name.set(target_folder_name)
+            self._files.create_folder.run(None)
+        
+        target_folder = self._files[target_folder_name]
+        publication = target_folder.publish(
+            revision_name=self.revision_name.get(),
+            source_path=self._folder.get_revision(self.revision_name.get()).path.get(),
+            comment=self.publish_comment.get()
+        )
+
+        # Cache source and target folder paths
+        self._source_folder_path = self._folder.get_revision(self.revision_name.get()).path.get()
+        self._target_folder_path = publication.path.get()
+        
+        # Get dimensions of the first image
+        image_paths = glob.glob("%s/*.png" % self._source_folder_path)
+        width, height = self._get_image_dimensions(image_paths[0])
+
+        # Cache dimensions
+        if height > width:
+            self._resize_format = "x%s" % min(int(0.5 * height), 3840)
+        else:
+            self._resize_format = "%sx" % min(int(0.5 * width), 3840)
+        
+        super(ResizeTrackedFolderImages, self).run(button)
+
+        self._files.touch()
+
+
 class TrackedFolder(TrackedFile):
 
     open = flow.Child(OpenTrackedFolderAction)
     history = flow.Child(TrackedFolderHistory)
+    resize_images = flow.Child(ResizeTrackedFolderImages).ui(group='Advanced')
     
     def get_name(self):
         return self.complete_name.get()
@@ -2064,7 +2187,7 @@ class TrackedFolder(TrackedFile):
             revisions.remove(working_copy.name())
 
         # Create new one
-        working_copy = revisions.add(user_name)
+        working_copy = revisions.add(user_name, ready_for_sync=False)
         working_copy.set_sync_status("Available")
 
         try:
@@ -2101,11 +2224,12 @@ class TrackedFolder(TrackedFile):
 
         return working_copy
 
-    def publish(self, revision_name=None, source_path=None, comment="", keep_editing=False):
+    def publish(self, revision_name=None, source_path=None, comment="", keep_editing=False, ready_for_sync=True):
         revisions = self.get_revisions()
-        head_revision = revisions.add(revision_name)
+        head_revision = revisions.add(revision_name, ready_for_sync=ready_for_sync)
         head_revision.comment.set(comment)
         head_revision.set_sync_status("Available")
+        head_revision_path = head_revision.get_path()
 
         if source_path:
             shutil.copytree(
@@ -2116,7 +2240,6 @@ class TrackedFolder(TrackedFile):
         else:
             working_copy = self.get_working_copy()
             working_copy_path = working_copy.get_path()
-            head_revision_path = head_revision.get_path()
 
             self.root().session().log_debug(
                 "Create head revision folder %s" % head_revision.path.get()
@@ -2200,6 +2323,12 @@ class TrackedFolder(TrackedFile):
             TrackedFile.compute_child_value(self, child_value)
 
 
+mapping = {r.name: r.index for r in TrackedFile._relations}
+for relation in TrackedFolder._relations:
+    if relation.name in ["open", "history"]:
+        relation.index = mapping.get(relation.name)
+
+
 class CreateFileAction(CreateFileSystemItemAction):
 
     format = flow.Param("blend", FileFormat)
