@@ -72,17 +72,22 @@ class KitsuAPIWrapper(flow.Object):
             self._config.project_name.get()
         )
     
-    def get_shot_data(self, shot_name, sequence_name):
-        sequence = self.get_sequence_data(sequence_name)
+    def get_shot_data(self, name, sequence):
+        if isinstance(sequence, str):
+            sequence = self.get_sequence_data(sequence)
+
         if not sequence:
             return None
         
         return gazu.shot.get_shot_by_name(
             sequence,
-            shot_name
+            name
         )
     
-    def get_shot_casting(self, shot):
+    def get_shot_casting(self, shot, sequence=None):
+        if isinstance(shot, str):
+            shot = self.get_shot_data(shot, sequence)
+
         return gazu.casting.get_shot_casting(shot)
     
     def get_sequence_data(self, name):
@@ -91,16 +96,21 @@ class KitsuAPIWrapper(flow.Object):
             name
         )
     
+    def get_sequence_casting(self, sequence):
+        if isinstance(sequence, str):
+            sequence = self.get_sequence_data(sequence)
+
+        return gazu.casting.get_sequence_casting(sequence)
+    
     def get_asset_data(self, name):
         return gazu.asset.get_asset_by_name(
             self._config.project_id.get(),
             name
         )
     
-    def get_asset_type(self, name):
-        asset = self.get_asset_data(name)
-        if not asset:
-            return None
+    def get_asset_type(self, asset):
+        if isinstance(asset, str):
+            asset = self.get_asset_data(asset)
 
         return gazu.asset.get_asset_type(asset["entity_type_id"])
 
