@@ -10,7 +10,7 @@ from kabaret.subprocess_manager.runner_factory import Runner, logger
 from kabaret.subprocess_manager.flow import RunAction
 
 
-CHOICES = ["blend", "kra", "png", "jpg", "txt", "abc", "mov", "psd", "aep", "zip", "mp4", "fbx", "ai"]
+CHOICES = ["blend", "kra", "png", "jpg", "txt", "abc", "mov", "psd", "aep", "zip", "mp4", "fbx", "ai", "json"]
 CHOICES_ICONS = {
             "blend": ("icons.libreflow", "blender"),
             "kra": ("icons.libreflow", "krita"),
@@ -25,6 +25,7 @@ CHOICES_ICONS = {
             "zip": ("icons.flow", "file"),
             "mp4": ("icons.gui", "youtube-logo"),
             "fbx": ("icons.libreflow", "fbx"),
+            "json": ("icons.libreflow", "json"),
         }
 
 
@@ -143,6 +144,13 @@ class EditFileRunner(BaseRunner):
             return os.environ[self.exec_env_var()]
 
 
+class ImageMagick(EditFileRunner):
+
+    @classmethod
+    def can_edit(cls, filename):
+        return True
+
+
 class Blender(EditFileRunner):
 
     ICON = ("icons.libreflow", "blender")
